@@ -23,6 +23,7 @@
 #include "CollectionView.h"
 #include "CollectionModel.h"
 #include "FBReader.h"
+#include "FBReaderActions.h"
 #include "BookInfoDialog.h"
 
 #include "../collection/BookList.h"
@@ -30,16 +31,15 @@
 static const std::string LIBRARY = "Library";
 
 CollectionView::CollectionView(FBReader &reader, shared_ptr<ZLPaintContext> context) : FBView(reader, context),
-	ShowTagsOption(ZLCategoryKey::LOOK_AND_FEEL, LIBRARY, "ShowTags", true),
 	ShowAllBooksTagOption(ZLCategoryKey::LOOK_AND_FEEL, LIBRARY, "ShowAllBooksTag", true),
 	myUpdateModel(true) {
-	setModel(new CollectionModel(*this, myCollection));
-	myShowTags = ShowTagsOption.value();
+	setModel(new CollectionModel(*this, myCollection), "");
+	myOrganizeByTags = organizeByTags();
 	myShowAllBooksList = ShowAllBooksTagOption.value();
 }
 
 CollectionView::~CollectionView() {
-	setModel(0);
+	setModel(0, "");
 }
 
 void CollectionView::updateModel() {
@@ -58,47 +58,79 @@ const std::string &CollectionView::caption() const {
 }
 
 void CollectionView::selectBook(BookDescriptionPtr book) {
+	mySelectedBook = book;
+
 	if (myUpdateModel) {
 		shared_ptr<ZLTextModel> oldModel = model();
-		setModel(0);
+		setModel(0, "");
 		((CollectionModel&)*oldModel).update();
-		setModel(oldModel);
+		setModel(oldModel, "");
 		myUpdateModel = false;
 	}
-	collectionModel().removeAllMarks();
-	const std::vector<int> &toSelect = collectionModel().paragraphNumbersByBook(book);
-	for (std::vector<int>::const_iterator it = toSelect.begin(); it != toSelect.end(); ++it) {
-		highlightParagraph(*it);
-	}
-	if (!toSelect.empty()) {
-		gotoParagraph(toSelect[toSelect.size() - 1]);
-		scrollPage(false, ZLTextView::SCROLL_PERCENTAGE, 40);
+
+	if (!book.isNull()) {
+		collectionModel().removeAllMarks();
+		const std::vector<int> &toSelect = collectionModel().paragraphIndicesByBook(book);
+		for (std::vector<int>::const_iterator it = toSelect.begin(); it != toSelect.end(); ++it) {
+			highlightParagraph(*it);
+		}
+		if (!toSelect.empty()) {
+			gotoParagraph(toSelect[toSelect.size() - 1]);
+			scrollPage(false, ZLTextView::SCROLL_PERCENTAGE, 40);
+		}
 	}
 }
 
 void CollectionView::paint() {
-	if ((myShowTags != ShowTagsOption.value()) ||
+	if ((myOrganizeByTags != organizeByTags()) ||
 			(myShowAllBooksList != ShowAllBooksTagOption.value())) {
-		myShowTags = ShowTagsOption.value();
+		myOrganizeByTags = organizeByTags();
 		myShowAllBooksList = ShowAllBooksTagOption.value();
 		myUpdateModel = true;
 	}
 	if (myUpdateModel) {
 		shared_ptr<ZLTextModel> oldModel = model();
-		setModel(0);
+		setModel(0, "");
 		((CollectionModel&)*oldModel).update();
-		setModel(oldModel);
+		setModel(oldModel, "");
 		myUpdateModel = false;
+		selectBook(mySelectedBook);
 	}
 	FBView::paint();
 }
 
+bool CollectionView::onStylusMove(int x, int y) {
+	if (FBView::onStylusMove(x, y)) {
+		return true;
+	}
+
+	const ZLTextElementArea *imageArea = elementByCoordinates(x, y);
+	if ((imageArea != 0) && (imageArea->Kind == ZLTextElement::IMAGE_ELEMENT)) {
+		fbreader().setHyperlinkCursor(true);
+		return true;
+	}
+
+	int index = paragraphIndexByCoordinate(y);
+	if (index != -1) {
+		BookDescriptionPtr book = collectionModel().bookByParagraphIndex(index);
+		if (!book.isNull()) {
+			fbreader().setHyperlinkCursor(true);
+			return true;
+		}
+	}
+
+	fbreader().setHyperlinkCursor(false);
+	return false;
+}
+
 bool CollectionView::_onStylusPress(int x, int y) {
+	fbreader().setHyperlinkCursor(false);
+
 	const ZLTextElementArea *imageArea = elementByCoordinates(x, y);
 	if ((imageArea != 0) && (imageArea->Kind == ZLTextElement::IMAGE_ELEMENT)) {
 		ZLTextWordCursor cursor = startCursor();
-		cursor.moveToParagraph(imageArea->ParagraphNumber);
-		cursor.moveTo(imageArea->TextElementNumber, 0);
+		cursor.moveToParagraph(imageArea->ParagraphIndex);
+		cursor.moveTo(imageArea->ElementIndex, 0);
 		const ZLTextElement &element = cursor.element();
 		if (element.kind() != ZLTextElement::IMAGE_ELEMENT) {
 			return false;
@@ -108,16 +140,16 @@ bool CollectionView::_onStylusPress(int x, int y) {
 		const std::string &id = imageElement.id();
 
 		if (id == CollectionModel::BookInfoImageId) {
-			editBookInfo(collectionModel().bookByParagraphNumber(imageArea->ParagraphNumber));
+			editBookInfo(collectionModel().bookByParagraphIndex(imageArea->ParagraphIndex));
 			return true;
 		} else if (id == CollectionModel::RemoveBookImageId) {
-			removeBook(collectionModel().bookByParagraphNumber(imageArea->ParagraphNumber));
+			removeBook(collectionModel().bookByParagraphIndex(imageArea->ParagraphIndex));
 			return true;
 		} else if (id == CollectionModel::RemoveTagImageId) {
-			removeTag(collectionModel().tagByParagraphNumber(imageArea->ParagraphNumber));
+			removeTag(collectionModel().tagByParagraphIndex(imageArea->ParagraphIndex));
 			return true;
 		} else if (id == CollectionModel::TagInfoImageId) {
-			editTagInfo(collectionModel().tagByParagraphNumber(imageArea->ParagraphNumber));
+			editTagInfo(collectionModel().tagByParagraphIndex(imageArea->ParagraphIndex));
 			return true;
 		} else {
 			return false;
@@ -129,7 +161,7 @@ bool CollectionView::_onStylusPress(int x, int y) {
 		return false;
 	}
 
-	BookDescriptionPtr book = collectionModel().bookByParagraphNumber(index);
+	BookDescriptionPtr book = collectionModel().bookByParagraphIndex(index);
 	if (!book.isNull()) {
 		fbreader().openBook(book);
 		fbreader().showBookTextView();
@@ -153,6 +185,10 @@ void CollectionView::removeBook(BookDescriptionPtr book) {
 		return;
 	}
 
+	if (book == mySelectedBook) {
+		mySelectedBook = 0;
+	}
+
 	CollectionModel &cModel = collectionModel();
 
 	ZLResourceKey boxKey("removeBookBox");
@@ -177,6 +213,7 @@ void CollectionView::removeBook(BookDescriptionPtr book) {
 			gotoParagraph(index);
 		}
 		rebuildPaintInfo(true);
+		selectBook(mySelectedBook);
 		application().refreshWindow();
 	}
 }
@@ -220,6 +257,7 @@ void CollectionView::removeTag(const std::string &tag) {
 		collectionModel().removeAllMarks();
 		myCollection.removeTag(tag, code == REMOVE_SUBTREE);
 		updateModel();
+		selectBook(mySelectedBook);
 		application().refreshWindow();
 	}
 }
@@ -227,3 +265,28 @@ void CollectionView::removeTag(const std::string &tag) {
 CollectionModel &CollectionView::collectionModel() {
 	return (CollectionModel&)*model();
 }
+
+class RebuildCollectionRunnable : public ZLRunnable {
+
+public:
+	RebuildCollectionRunnable(CollectionView &view) : myView(view) {}
+	void run() {
+		myView.updateModel();
+		myView.collection().authors();
+	}
+
+private:
+	CollectionView &myView;
+};
+
+void CollectionView::openWithBook(BookDescriptionPtr book) {
+	RebuildCollectionRunnable runnable(*this);
+	ZLDialogManager::instance().wait(ZLResourceKey("loadingBookList"), runnable);
+	if (book != 0) {
+		selectBook(book);
+	}
+}
+
+bool CollectionView::organizeByTags() const {
+	return ZLStringOption(ZLCategoryKey::LOOK_AND_FEEL, "ToggleButtonGroup", "booksOrder", "").value() == ActionCode::ORGANIZE_BOOKS_BY_TAG;
+}
