@@ -37,7 +37,6 @@ public:
 	static const std::string SpecialTagNoTagsBooks;
 
 public:
-	ZLBooleanOption ShowTagsOption;
 	ZLBooleanOption ShowAllBooksTagOption;
 
 public:
@@ -46,16 +45,21 @@ public:
 	const std::string &caption() const;
 
 	bool _onStylusPress(int x, int y);
+	bool onStylusMove(int x, int y);
 
 	void paint();
 	void updateModel();
 	void synchronizeModel();
 
-	void selectBook(BookDescriptionPtr book);
+	void openWithBook(BookDescriptionPtr book);
 
 	BookCollection &collection();
 
+	bool organizeByTags() const;
+
 private:
+	void selectBook(BookDescriptionPtr book);
+
 	CollectionModel &collectionModel();
 
 	void editBookInfo(BookDescriptionPtr book);
@@ -68,10 +72,11 @@ private:
 
 private:
 	BookCollection myCollection;
+	BookDescriptionPtr mySelectedBook;
 	bool myTreeStateIsFrozen;
 	bool myUpdateModel;
 
-	bool myShowTags;
+	bool myOrganizeByTags;
 	bool myShowAllBooksList;
 };
 
