@@ -29,11 +29,13 @@
 #include <ZLApplication.h>
 #include <ZLKeyBindings.h>
 
+#include "../collection/BookCollection.h"
 #include "../description/BookDescription.h"
 #include "../external/ProgramCollection.h"
 
 class ZLViewWidget;
 class ZLMessageHandler;
+class ZLPopupData;
 
 class BookModel;
 class BookTextView;
@@ -48,6 +50,8 @@ public:
 	// returns true if description was found or error message was shown
 	static bool createDescription(const std::string &fileName, BookDescriptionPtr &description);
 
+	static const std::string PageIndexParameter;
+
 public:
 	enum ViewMode {
 		UNDEFINED_MODE = 0,
@@ -57,6 +61,7 @@ public:
 		BOOKMARKS_MODE = 1 << 3,
 		BOOK_COLLECTION_MODE = 1 << 4,
 		RECENT_BOOKS_MODE = 1 << 5,
+		ALL_MODES = 0x3F
 	};
 
 	struct ScrollingOptions {
@@ -88,7 +93,7 @@ public:
 	~FBReader();
 
 	void setMode(ViewMode mode);
-	ViewMode getMode() const;
+	ViewMode mode() const;
 
 private:
 	void initWindow();
@@ -103,9 +108,11 @@ private:
 	std::string helpFileName(const std::string &language) const;
 	void openFile(const std::string &fileName);
 
+	bool isViewFinal() const;
+
 public:
 	ZLKeyBindings &keyBindings();
-	ZLKeyBindings &keyBindings(ZLViewWidget::Angle angle);
+	ZLKeyBindings &keyBindings(ZLView::Angle angle);
 
 	bool isDictionarySupported() const;
 	void openInDictionary(const std::string &word);
@@ -118,6 +125,9 @@ public:
 	void showBookTextView();
 	void openBook(BookDescriptionPtr description);
 
+	RecentBooks &recentBooks();
+	const RecentBooks &recentBooks() const;
+
 private:
 	shared_ptr<ProgramCollection> dictionaryCollection() const;
 
@@ -136,6 +146,7 @@ private:
 	shared_ptr<ZLView> myContentsView;	
 	shared_ptr<ZLView> myCollectionView;	
 	shared_ptr<ZLView> myRecentBooksView;	
+	shared_ptr<ZLPopupData> myRecentBooksPopupData;	
 
 	ZLTime myLastScrollingTime;
 
@@ -153,10 +164,18 @@ private:
 
 	shared_ptr<ZLMessageHandler> myOpenFileHandler;
 
+	RecentBooks myRecentBooks;
+
+	enum {
+		RETURN_TO_TEXT_MODE,
+		UNFULLSCREEN
+	} myActionOnCancel;
+
 friend class OpenFileHandler;
 
 friend class OptionsDialog;
 friend class FBView;
+friend class BookTextView;
 
 //friend class ShowCollectionAction;
 friend class ShowHelpAction;
@@ -183,6 +202,7 @@ friend class GotoNextTOCSectionAction;
 friend class GotoPreviousTOCSectionAction;
 //friend class GotoPageNumber;
 friend class SelectionAction;
+friend class FBFullscreenAction;
 };
 
 #endif /* __FBREADER_H__ */
