@@ -17,6 +17,8 @@
  * 02110-1301, USA.
  */
 
+#include <stdlib.h>
+
 #include <ZLDialogManager.h>
 #include <ZLDialog.h>
 #include <ZLOptionsDialog.h>
@@ -29,7 +31,7 @@
 #include "FBReaderActions.h"
 #include "BookTextView.h"
 #include "ContentsView.h"
-#include "RecentBooksView.h"
+#include "CollectionView.h"
 #include "FBFileHandler.h"
 
 #include "../bookmodel/BookModel.h"
@@ -41,8 +43,8 @@ FBAction::FBAction(FBReader &fbreader) : myFBReader(fbreader) {
 ModeDependentAction::ModeDependentAction(FBReader &fbreader, int visibleInModes) : FBAction(fbreader), myVisibleInModes(visibleInModes) {
 }
 
-bool ModeDependentAction::isVisible() {
-	return fbreader().getMode() & myVisibleInModes;
+bool ModeDependentAction::isVisible() const {
+	return fbreader().mode() & myVisibleInModes;
 }
 
 SetModeAction::SetModeAction(FBReader &fbreader, FBReader::ViewMode modeToSet, int visibleInModes) : ModeDependentAction(fbreader, visibleInModes), myModeToSet(modeToSet) {
@@ -52,7 +54,7 @@ void SetModeAction::run() {
 	fbreader().setMode(myModeToSet);
 }
 
-ShowHelpAction::ShowHelpAction(FBReader &fbreader) : FBAction(fbreader) {
+ShowHelpAction::ShowHelpAction(FBReader &fbreader) : ModeDependentAction(fbreader, FBReader::BOOK_TEXT_MODE | FBReader::FOOTNOTE_MODE | FBReader::BOOK_COLLECTION_MODE) {
 }
 
 void ShowHelpAction::run() {
@@ -69,7 +71,7 @@ void ShowHelpAction::run() {
 	}
 }
 
-ShowOptionsDialogAction::ShowOptionsDialogAction(FBReader &fbreader) : FBAction(fbreader) {
+ShowOptionsDialogAction::ShowOptionsDialogAction(FBReader &fbreader) : ModeDependentAction(fbreader, FBReader::BOOK_TEXT_MODE | FBReader::FOOTNOTE_MODE | FBReader::BOOK_COLLECTION_MODE) {
 }
 
 void ShowOptionsDialogAction::run() {
@@ -77,26 +79,18 @@ void ShowOptionsDialogAction::run() {
 	OptionsDialog(f).dialog().run();
 }
 
-ShowContentsAction::ShowContentsAction(FBReader &fbreader) : FBAction(fbreader) {
+ShowContentsAction::ShowContentsAction(FBReader &fbreader) : ModeDependentAction(fbreader, FBReader::BOOK_TEXT_MODE | FBReader::FOOTNOTE_MODE) {
 }
 
-bool ShowContentsAction::isVisible() {
-	if (((ContentsView&)*fbreader().myContentsView).isEmpty()) {
-		return false;
-	}
-	FBReader::ViewMode mode = fbreader().getMode();
-	return (mode == FBReader::BOOK_TEXT_MODE) || (mode == FBReader::FOOTNOTE_MODE);
+bool ShowContentsAction::isEnabled() const {
+	return isVisible() && !((ContentsView&)*fbreader().myContentsView).isEmpty();
 }
 
 void ShowContentsAction::run() {
 	fbreader().setMode(FBReader::CONTENTS_MODE);
 }
 
-AddBookAction::AddBookAction(FBReader &fbreader) : FBAction(fbreader) {
-}
-
-bool AddBookAction::isVisible() {
-	return fbreader().getMode() != FBReader::FOOTNOTE_MODE;
+AddBookAction::AddBookAction(FBReader &fbreader, int visibleInModes) : ModeDependentAction(fbreader, visibleInModes) {
 }
 
 void AddBookAction::run() {
@@ -105,8 +99,13 @@ void AddBookAction::run() {
 		BookDescriptionPtr description = handler.description();
 		const std::string &fileName = description->fileName();
 		if (!description.isNull() && fbreader().runBookInfoDialog(fileName)) {
-			fbreader().openFile(fileName);
-			fbreader().setMode(FBReader::BOOK_TEXT_MODE);
+			if (fbreader().mode() == FBReader::BOOK_COLLECTION_MODE) {
+				fbreader().collectionView().openWithBook(description);
+			} else {
+				fbreader().openFile(fileName);
+				fbreader().setMode(FBReader::BOOK_TEXT_MODE);
+			}
+			fbreader().refreshWindow();
 		}
 	}
 }
@@ -132,7 +131,7 @@ void ScrollToEndOfTextAction::run() {
 	fbreader().bookTextView().scrollToEndOfText();
 }
 
-ShowBookInfoAction::ShowBookInfoAction(FBReader &fbreader) : ModeDependentAction(fbreader, FBReader::BOOK_TEXT_MODE | FBReader::CONTENTS_MODE | FBReader::FOOTNOTE_MODE) {
+ShowBookInfoAction::ShowBookInfoAction(FBReader &fbreader) : ModeDependentAction(fbreader, FBReader::BOOK_TEXT_MODE | FBReader::FOOTNOTE_MODE) {
 }
 
 void ShowBookInfoAction::run() {
@@ -142,16 +141,17 @@ void ShowBookInfoAction::run() {
 	}
 }
 
-UndoAction::UndoAction(FBReader &fbreader) : FBAction(fbreader) {
+UndoAction::UndoAction(FBReader &fbreader, int visibleInModes) : ModeDependentAction(fbreader, visibleInModes) {
 }
 
-bool UndoAction::isEnabled() {
-	return (fbreader().getMode() != FBReader::BOOK_TEXT_MODE) ||
-					fbreader().bookTextView().canUndoPageMove();
+bool UndoAction::isEnabled() const {
+	return isVisible() &&
+				 ((fbreader().mode() != FBReader::BOOK_TEXT_MODE) ||
+					fbreader().bookTextView().canUndoPageMove());
 }
 
 void UndoAction::run() {
-	if (fbreader().getMode() == FBReader::BOOK_TEXT_MODE) {
+	if (fbreader().mode() == FBReader::BOOK_TEXT_MODE) {
 		fbreader().bookTextView().undoPageMove();
 	} else {
 		fbreader().restorePreviousMode();
@@ -161,7 +161,7 @@ void UndoAction::run() {
 RedoAction::RedoAction(FBReader &fbreader) : ModeDependentAction(fbreader, FBReader::BOOK_TEXT_MODE) {
 }
 
-bool RedoAction::isEnabled() {
+bool RedoAction::isEnabled() const {
 	return isVisible() && fbreader().bookTextView().canRedoPageMove();
 }
 
@@ -172,7 +172,7 @@ void RedoAction::run() {
 ScrollingAction::ScrollingAction(FBReader &fbreader, const FBReader::ScrollingOptions &options, bool forward) : FBAction(fbreader), myOptions(options), myForward(forward) {
 }
 
-bool ScrollingAction::isEnabled() {
+bool ScrollingAction::isEnabled() const {
 	return
 		(&myOptions != &fbreader().TapScrollingOptions) ||
 		fbreader().EnableTapScrollingOption.value();
@@ -210,6 +210,15 @@ void ScrollingAction::run() {
 ChangeFontSizeAction::ChangeFontSizeAction(FBReader &fbreader, int delta) : FBAction(fbreader), myDelta(delta) {
 }
 
+bool ChangeFontSizeAction::isEnabled() const {
+	ZLIntegerRangeOption &option = ZLTextStyleCollection::instance().baseStyle().FontSizeOption;
+	if (myDelta < 0) {
+		return option.value() > option.minValue();
+	} else {
+		return option.value() < option.maxValue();
+	}
+}
+
 void ChangeFontSizeAction::run() {
 	ZLIntegerRangeOption &option = ZLTextStyleCollection::instance().baseStyle().FontSizeOption;
 	option.setValue(option.value() + myDelta);
@@ -220,15 +229,15 @@ void ChangeFontSizeAction::run() {
 OpenPreviousBookAction::OpenPreviousBookAction(FBReader &fbreader) : FBAction(fbreader) {
 }
 
-bool OpenPreviousBookAction::isVisible() {
-	if ((fbreader().getMode() != FBReader::BOOK_TEXT_MODE) && (fbreader().getMode() != FBReader::CONTENTS_MODE)) {
+bool OpenPreviousBookAction::isVisible() const {
+	if ((fbreader().mode() != FBReader::BOOK_TEXT_MODE) && (fbreader().mode() != FBReader::CONTENTS_MODE)) {
 		return false;
 	}
-	return ((RecentBooksView&)*fbreader().myRecentBooksView).lastBooks().books().size() > 1;
+	return fbreader().recentBooks().books().size() > 1;
 }
 
 void OpenPreviousBookAction::run() {
-	Books books = ((RecentBooksView&)*fbreader().myRecentBooksView).lastBooks().books();
+	Books books = fbreader().recentBooks().books();
 	fbreader().openBook(books[1]);
 	fbreader().refreshWindow();
 	fbreader().resetWindowCaption();
@@ -238,11 +247,27 @@ CancelAction::CancelAction(FBReader &fbreader) : FBAction(fbreader) {
 }
 
 void CancelAction::run() {
-	if (fbreader().getMode() != FBReader::BOOK_TEXT_MODE) {
-		fbreader().restorePreviousMode();
-	} else if (fbreader().isFullscreen()) {
-		fbreader().setFullscreen(false);
-	} else if (fbreader().QuitOnCancelOption.value()) {
+	switch (fbreader().myActionOnCancel) {
+		case FBReader::UNFULLSCREEN:
+			if (fbreader().isFullscreen()) {
+				fbreader().setFullscreen(false);
+				return;
+			} else if (fbreader().mode() != FBReader::BOOK_TEXT_MODE) {
+				fbreader().restorePreviousMode();
+				return;
+			}
+			break;
+		case FBReader::RETURN_TO_TEXT_MODE:
+			if (fbreader().mode() != FBReader::BOOK_TEXT_MODE) {
+				fbreader().restorePreviousMode();
+				return;
+			} else if (fbreader().isFullscreen()) {
+				fbreader().setFullscreen(false);
+				return;
+			}
+			break;
+	}
+	if (fbreader().QuitOnCancelOption.value()) {
 		fbreader().quit();
 	}
 }
@@ -250,10 +275,30 @@ void CancelAction::run() {
 ToggleIndicatorAction::ToggleIndicatorAction(FBReader &fbreader) : FBAction(fbreader) {
 }
 
+bool ToggleIndicatorAction::isVisible() const {
+	ZLIntegerRangeOption &option = FBView::commonIndicatorInfo().TypeOption;
+	switch (option.value()) {
+		case FBIndicatorStyle::FB_INDICATOR:
+		case FBIndicatorStyle::NONE:
+			return true;
+	}
+	return false;
+}
+
 void ToggleIndicatorAction::run() {
-	ZLBooleanOption &option = FBView::commonIndicatorInfo().ShowOption;
-	option.setValue(!option.value());
-	fbreader().refreshWindow();
+	ZLIntegerRangeOption &option = FBView::commonIndicatorInfo().TypeOption;
+	switch (option.value()) {
+		case FBIndicatorStyle::OS_SCROLLBAR:
+			break;
+		case FBIndicatorStyle::FB_INDICATOR:
+			option.setValue(FBIndicatorStyle::NONE);
+			fbreader().refreshWindow();
+			break;
+		case FBIndicatorStyle::NONE:
+			option.setValue(FBIndicatorStyle::FB_INDICATOR);
+			fbreader().refreshWindow();
+			break;
+	}
 }
 
 QuitAction::QuitAction(FBReader &fbreader) : FBAction(fbreader) {
@@ -266,8 +311,8 @@ void QuitAction::run() {
 GotoNextTOCSectionAction::GotoNextTOCSectionAction(FBReader &fbreader) : FBAction(fbreader) {
 }
 
-bool GotoNextTOCSectionAction::isVisible() {
-	if (fbreader().getMode() != FBReader::BOOK_TEXT_MODE) {
+bool GotoNextTOCSectionAction::isVisible() const {
+	if (fbreader().mode() != FBReader::BOOK_TEXT_MODE) {
 		return false;
 	}
 	const ContentsView &contentsView = (const ContentsView&)*fbreader().myContentsView;
@@ -275,7 +320,7 @@ bool GotoNextTOCSectionAction::isVisible() {
 	return !model.isNull() && (model->paragraphsNumber() > 1);
 }
 
-bool GotoNextTOCSectionAction::isEnabled() {
+bool GotoNextTOCSectionAction::isEnabled() const {
 	const ContentsView &contentsView = (const ContentsView&)*fbreader().myContentsView;
 	shared_ptr<ZLTextModel> model = contentsView.model();
 	return !model.isNull() && ((int)contentsView.currentTextViewParagraph() < (int)model->paragraphsNumber() - 1);
@@ -293,8 +338,8 @@ void GotoNextTOCSectionAction::run() {
 GotoPreviousTOCSectionAction::GotoPreviousTOCSectionAction(FBReader &fbreader) : FBAction(fbreader) {
 }
 
-bool GotoPreviousTOCSectionAction::isVisible() {
-	if (fbreader().getMode() != FBReader::BOOK_TEXT_MODE) {
+bool GotoPreviousTOCSectionAction::isVisible() const {
+	if (fbreader().mode() != FBReader::BOOK_TEXT_MODE) {
 		return false;
 	}
 	const ContentsView &contentsView = (const ContentsView&)*fbreader().myContentsView;
@@ -302,7 +347,7 @@ bool GotoPreviousTOCSectionAction::isVisible() {
 	return !model.isNull() && (model->paragraphsNumber() > 1);
 }
 
-bool GotoPreviousTOCSectionAction::isEnabled() {
+bool GotoPreviousTOCSectionAction::isEnabled() const {
 	const ContentsView &contentsView = (const ContentsView&)*fbreader().myContentsView;
 	shared_ptr<ZLTextModel> model = contentsView.model();
 	if (model.isNull()) {
@@ -318,7 +363,7 @@ bool GotoPreviousTOCSectionAction::isEnabled() {
 		if (cursor.isNull()) {
 			return false;
 		}
-		if (cursor.wordNumber() > 0) {
+		if (cursor.elementIndex() > 0) {
 			return true;
 		}
 		return
@@ -336,7 +381,7 @@ void GotoPreviousTOCSectionAction::run() {
 	int reference = contentsModel.reference(((const ZLTextTreeParagraph*)contentsModel[current]));
 	const ZLTextWordCursor &cursor = fbreader().bookTextView().startCursor();
 	if (!cursor.isNull() &&
-			(cursor.wordNumber() == 0) &&
+			(cursor.elementIndex() == 0) &&
 			(reference == (int)cursor.paragraphCursor().index())) {
 		reference = contentsModel.reference(((const ZLTextTreeParagraph*)contentsModel[current - 1]));
 	}
@@ -344,49 +389,72 @@ void GotoPreviousTOCSectionAction::run() {
 	fbreader().refreshWindow();
 }
 
-GotoPageNumber::GotoPageNumber(FBReader &fbreader) : ModeDependentAction(fbreader, FBReader::BOOK_TEXT_MODE) {
+GotoPageNumber::GotoPageNumber(FBReader &fbreader, const std::string &parameter) : ModeDependentAction(fbreader, FBReader::BOOK_TEXT_MODE), myParameter(parameter) {
 }
 
-bool GotoPageNumber::isEnabled() {
+bool GotoPageNumber::isVisible() const {
+	return
+		ModeDependentAction::isVisible() &&
+		(myParameter.empty() || !fbreader().bookTextView().hasMultiSectionModel());
+}
+
+bool GotoPageNumber::isEnabled() const {
 	return ModeDependentAction::isEnabled() && (fbreader().bookTextView().pageNumber() > 1);
 }
 
 void GotoPageNumber::run() {
-	shared_ptr<ZLDialog> gotoPageDialog = ZLDialogManager::instance().createDialog(ZLResourceKey("gotoPageDialog"));
+	int pageIndex = 0;
 
-	const int pageNumber = fbreader().bookTextView().pageNumber();
-	ZLIntegerRangeOption pageNumberOption(ZLCategoryKey::CONFIG, "gotoPageDialog", "Number", 0, pageNumber, pageNumber);
-	gotoPageDialog->addOption(ZLResourceKey("pageNumber"), new ZLSimpleSpinOptionEntry(pageNumberOption, 1));
-	gotoPageDialog->addButton(ZLDialogManager::OK_BUTTON, true);
-	gotoPageDialog->addButton(ZLDialogManager::CANCEL_BUTTON, false);
+	if (!myParameter.empty()) {
+		const std::string value = fbreader().visualParameter(myParameter);
+		if (value.empty()) {
+			return;
+		}
+		pageIndex = atoi(value.c_str());
+	} else {
+		shared_ptr<ZLDialog> gotoPageDialog = ZLDialogManager::instance().createDialog(ZLResourceKey("gotoPageDialog"));
 
-	if (gotoPageDialog->run()) {
-		gotoPageDialog->acceptValues();
-		fbreader().bookTextView().gotoPage(pageNumberOption.value());
-		fbreader().refreshWindow();
+		const int pageNumber = fbreader().bookTextView().pageNumber();
+		ZLIntegerRangeOption pageIndexOption(ZLCategoryKey::CONFIG, "gotoPageDialog", "Index", 0, pageNumber, pageIndex);
+		gotoPageDialog->addOption(ZLResourceKey("pageNumber"), new ZLSimpleSpinOptionEntry(pageIndexOption, 1));
+		gotoPageDialog->addButton(ZLDialogManager::OK_BUTTON, true);
+		gotoPageDialog->addButton(ZLDialogManager::CANCEL_BUTTON, false);
+		pageIndex = pageIndexOption.value();
+		if (gotoPageDialog->run()) {
+			gotoPageDialog->acceptValues();
+		} else {
+			return;
+		}
 	}
+
+	fbreader().bookTextView().gotoPage(pageIndex);
+	fbreader().refreshWindow();
 }
 
-SelectionAction::SelectionAction(FBReader &fbreader) : FBAction(fbreader) {
+SelectionAction::SelectionAction(FBReader &fbreader) : ModeDependentAction(fbreader, FBReader::BOOK_TEXT_MODE | FBReader::FOOTNOTE_MODE) {
 }
 
-bool SelectionAction::isVisible() {
-	return !fbreader().currentView().isNull();
+bool SelectionAction::isVisible() const {
+	return textView().isSelectionEnabled();
 }
 
-bool SelectionAction::isEnabled() {
-	return isVisible() && !textView().selectionModel().getText().empty();
+bool SelectionAction::isEnabled() const {
+	return isVisible() && !textView().selectionModel().text().empty();
 }
 
 ZLTextView &SelectionAction::textView() {
 	return (ZLTextView&)*fbreader().currentView();
 }
 
+const ZLTextView &SelectionAction::textView() const {
+	return (ZLTextView&)*fbreader().currentView();
+}
+
 CopySelectedTextAction::CopySelectedTextAction(FBReader &fbreader) : SelectionAction(fbreader) {
 }
 
-bool CopySelectedTextAction::isVisible() {
-	return SelectionAction::isVisible() && ZLDialogManager::instance().isClipboardSupported(ZLDialogManager::CLIPBOARD_MAIN);
+bool CopySelectedTextAction::isVisible() const {
+	return ModeDependentAction::isVisible() && ZLDialogManager::instance().isClipboardSupported(ZLDialogManager::CLIPBOARD_MAIN);
 }
 
 void CopySelectedTextAction::run() {
@@ -396,12 +464,12 @@ void CopySelectedTextAction::run() {
 OpenSelectedTextInDictionaryAction::OpenSelectedTextInDictionaryAction(FBReader &fbreader) : SelectionAction(fbreader) {
 }
 
-bool OpenSelectedTextInDictionaryAction::isVisible() {
+bool OpenSelectedTextInDictionaryAction::isVisible() const {
 	return SelectionAction::isVisible() && fbreader().isDictionarySupported();
 }
 
 void OpenSelectedTextInDictionaryAction::run() {
-	fbreader().openInDictionary(textView().selectionModel().getText());
+	fbreader().openInDictionary(textView().selectionModel().text());
 }
 
 ClearSelectionAction::ClearSelectionAction(FBReader &fbreader) : SelectionAction(fbreader) {
@@ -411,3 +479,21 @@ void ClearSelectionAction::run() {
 	textView().selectionModel().clear();
 	fbreader().refreshWindow();
 }
+
+TouchScreenModeAction::TouchScreenModeAction(FBReader &fbreader) : ModeDependentAction(fbreader, FBReader::BOOK_TEXT_MODE) {
+}
+
+void TouchScreenModeAction::run() {
+	fbreader().bookTextView().selectionModel().clear();
+	fbreader().refreshWindow();
+}
+
+FBFullscreenAction::FBFullscreenAction(FBReader &fbreader) : ZLApplication::FullscreenAction(fbreader), myFBReader(fbreader) {
+}
+
+void FBFullscreenAction::run() {
+	if (!myFBReader.isFullscreen()) {
+		myFBReader.myActionOnCancel = FBReader::UNFULLSCREEN;
+	}
+	FullscreenAction::run();
+}
