@@ -29,6 +29,7 @@ class ZLTextView;
 class ActionCode {
 
 public:
+	static const std::string SHOW_READING;
 	static const std::string SHOW_COLLECTION;
 	static const std::string SHOW_OPTIONS;
 	static const std::string UNDO;
@@ -54,7 +55,10 @@ public:
 	static const std::string SHOW_HIDE_POSITION_INDICATOR;
 	static const std::string TOGGLE_FULLSCREEN;
 	static const std::string FULLSCREEN_ON;
+	static const std::string FULLSCREEN_OFF;
 	static const std::string ADD_BOOK;
+	static const std::string ADD_BOOK_TO_LIBRARY;
+	static const std::string OPEN_FILE;
 	static const std::string SHOW_BOOK_INFO;
 	static const std::string SHOW_HELP;
 	static const std::string ROTATE_SCREEN;
@@ -67,6 +71,11 @@ public:
 	static const std::string CLEAR_SELECTION;
 	static const std::string OPEN_SELECTED_TEXT_IN_DICTIONARY;
 	static const std::string GOTO_PAGE_NUMBER;
+	static const std::string GOTO_PAGE_NUMBER_WITH_PARAMETER;
+	static const std::string HAND_TOOL;
+	static const std::string SELECT_TEXT;
+	static const std::string ORGANIZE_BOOKS_BY_AUTHOR;
+	static const std::string ORGANIZE_BOOKS_BY_TAG;
 
 private:
 	ActionCode();
@@ -77,6 +86,7 @@ class FBAction : public ZLApplication::Action {
 protected:
 	FBAction(FBReader &fbreader);
 	FBReader &fbreader();
+	const FBReader &fbreader() const;
 
 private:
 	FBReader &myFBReader;
@@ -88,7 +98,7 @@ protected:
 	ModeDependentAction(FBReader &fbreader, int visibleInModes);
 
 public:
-	bool isVisible();
+	bool isVisible() const;
 
 private:
 	int myVisibleInModes;
@@ -104,33 +114,32 @@ private:
 	FBReader::ViewMode myModeToSet;
 };
 
-class ShowHelpAction : public FBAction {
+class ShowHelpAction : public ModeDependentAction {
 
 public:
 	ShowHelpAction(FBReader &fbreader);
 	void run();
 };
 
-class ShowOptionsDialogAction : public FBAction {
+class ShowOptionsDialogAction : public ModeDependentAction {
 
 public:
 	ShowOptionsDialogAction(FBReader &fbreader);
 	void run();
 };
 
-class ShowContentsAction : public FBAction {
+class ShowContentsAction : public ModeDependentAction {
 
 public:
 	ShowContentsAction(FBReader &fbreader);
-	bool isVisible();
+	bool isEnabled() const;
 	void run();
 };
 
-class AddBookAction : public FBAction {
+class AddBookAction : public ModeDependentAction {
 
 public:
-	AddBookAction(FBReader &fbreader);
-	bool isVisible();
+	AddBookAction(FBReader &fbreader, int visibleInModes);
 	void run();
 };
 
@@ -145,7 +154,7 @@ class ScrollingAction : public FBAction {
 
 public:
 	ScrollingAction(FBReader &fbreader, const FBReader::ScrollingOptions &options, bool forward);
-	bool isEnabled();
+	bool isEnabled() const;
 	bool useKeyDelay() const;
 	void run();
 
@@ -175,11 +184,11 @@ public:
 	void run();
 };
 
-class UndoAction : public FBAction {
+class UndoAction : public ModeDependentAction {
 
 public:
-	UndoAction(FBReader &fbreader);
-	bool isEnabled();
+	UndoAction(FBReader &fbreader, int visibleInModes);
+	bool isEnabled() const;
 	void run();
 };
 
@@ -187,7 +196,7 @@ class RedoAction : public ModeDependentAction {
 
 public:
 	RedoAction(FBReader &fbreader);
-	bool isEnabled();
+	bool isEnabled() const;
 	void run();
 };
 
@@ -195,7 +204,7 @@ class SearchAction : public FBAction {
 
 public:
 	SearchAction(FBReader &fbreader);
-	bool isVisible();
+	bool isVisible() const;
 	void run();
 
 private:
@@ -212,7 +221,7 @@ class FindNextAction : public FBAction {
 
 public:
 	FindNextAction(FBReader &fbreader);
-	bool isEnabled();
+	bool isEnabled() const;
 	void run();
 };
 
@@ -220,7 +229,7 @@ class FindPreviousAction : public FBAction {
 
 public:
 	FindPreviousAction(FBReader &fbreader);
-	bool isEnabled();
+	bool isEnabled() const;
 	void run();
 };
 
@@ -228,6 +237,7 @@ class ChangeFontSizeAction : public FBAction {
 
 public:
 	ChangeFontSizeAction(FBReader &fbreader, int delta);
+	bool isEnabled() const;
 	void run();
 
 private:
@@ -245,6 +255,7 @@ class ToggleIndicatorAction : public FBAction {
 
 public:
 	ToggleIndicatorAction(FBReader &fbreader);
+	bool isVisible() const;
 	void run();
 };
 
@@ -259,7 +270,7 @@ class OpenPreviousBookAction : public FBAction {
 
 public:
 	OpenPreviousBookAction(FBReader &fbreader);
-	bool isVisible();
+	bool isVisible() const;
 	void run();
 };
 
@@ -267,8 +278,8 @@ class GotoNextTOCSectionAction : public FBAction {
 
 public:
 	GotoNextTOCSectionAction(FBReader &fbreader);
-	bool isVisible();
-	bool isEnabled();
+	bool isVisible() const;
+	bool isEnabled() const;
 	void run();
 };
 
@@ -276,35 +287,40 @@ class GotoPreviousTOCSectionAction : public FBAction {
 
 public:
 	GotoPreviousTOCSectionAction(FBReader &fbreader);
-	bool isVisible();
-	bool isEnabled();
+	bool isVisible() const;
+	bool isEnabled() const;
 	void run();
 };
 
 class GotoPageNumber : public ModeDependentAction {
 
 public:
-	GotoPageNumber(FBReader &fbreader);
-	bool isEnabled();
+	GotoPageNumber(FBReader &fbreader, const std::string &parameter);
+	bool isVisible() const;
+	bool isEnabled() const;
 	void run();
+
+private:
+	const std::string myParameter;
 };
 
-class SelectionAction : public FBAction {
+class SelectionAction : public ModeDependentAction {
 
 public:
 	SelectionAction(FBReader &fbreader);
-	bool isVisible();
-	bool isEnabled();
+	bool isVisible() const;
+	bool isEnabled() const;
 
 protected:
 	ZLTextView &textView();
+	const ZLTextView &textView() const;
 };
 
 class CopySelectedTextAction : public SelectionAction {
 
 public:
 	CopySelectedTextAction(FBReader &fbreader);
-	bool isVisible();
+	bool isVisible() const;
 	void run();
 };
 
@@ -312,7 +328,7 @@ class OpenSelectedTextInDictionaryAction : public SelectionAction {
 
 public:
 	OpenSelectedTextInDictionaryAction(FBReader &fbreader);
-	bool isVisible();
+	bool isVisible() const;
 	void run();
 };
 
@@ -323,8 +339,36 @@ public:
 	void run();
 };
 
+class TouchScreenModeAction : public ModeDependentAction {
+
+public:
+	TouchScreenModeAction(FBReader &fbreader);
+	void run();
+};
+
+class FBFullscreenAction : public ZLApplication::FullscreenAction {
+
+public:
+	FBFullscreenAction(FBReader &fbreader);
+	void run();
+
+private:
+	FBReader &myFBReader;
+};
+
+class BooksOrderAction : public ModeDependentAction {
+
+public:
+	BooksOrderAction(FBReader &fbreader);
+	void run();
+};
+
 inline FBReader &FBAction::fbreader() {
 	return myFBReader;
 }
 
+inline const FBReader &FBAction::fbreader() const {
+	return myFBReader;
+}
+
 #endif /* __FBREADERACTIONS_H__ */
