@@ -35,7 +35,7 @@ FBMargins::FBMargins() :
 static const std::string INDICATOR = "Indicator";
 
 FBIndicatorStyle::FBIndicatorStyle() :
-	ShowOption(ZLCategoryKey::LOOK_AND_FEEL, INDICATOR, "Show", true),
+	TypeOption(ZLCategoryKey::LOOK_AND_FEEL, INDICATOR, "Type", 0, 2, ZLTextPositionIndicatorInfo::OS_SCROLLBAR),
 	IsSensitiveOption(ZLCategoryKey::LOOK_AND_FEEL, INDICATOR, "TouchSensitive", true),
 	ShowTextPositionOption(ZLCategoryKey::LOOK_AND_FEEL, INDICATOR, "PositionText", true),
 	ShowTimeOption(ZLCategoryKey::LOOK_AND_FEEL, INDICATOR, "Time", true),
@@ -45,8 +45,8 @@ FBIndicatorStyle::FBIndicatorStyle() :
 	FontSizeOption(ZLCategoryKey::LOOK_AND_FEEL, INDICATOR, "FontSize", 4, 72, 14) {
 }
 
-bool FBIndicatorStyle::isVisible() const {
-	return ShowOption.value();
+ZLTextPositionIndicatorInfo::Type FBIndicatorStyle::type() const {
+	return (ZLTextPositionIndicatorInfo::Type)TypeOption.value();
 }
 
 bool FBIndicatorStyle::isSensitive() const {
@@ -79,7 +79,7 @@ int FBIndicatorStyle::fontSize() const {
 
 shared_ptr<ZLTextPositionIndicatorInfo> FBView::ourIndicatorInfo;
 shared_ptr<FBMargins> FBView::ourMargins;
-shared_ptr<ZLBooleanOption> FBView::ourSelectionOption;
+//shared_ptr<ZLBooleanOption> FBView::ourSelectionOption;
 
 FBIndicatorStyle& FBView::commonIndicatorInfo() {
 	if (ourIndicatorInfo.isNull()) {
@@ -148,24 +148,24 @@ std::string FBView::word(const ZLTextElementArea &area) const {
 
 	if (area.Kind == ZLTextElement::WORD_ELEMENT) {
 		ZLTextWordCursor cursor = startCursor();
-		cursor.moveToParagraph(area.ParagraphNumber);
-		cursor.moveTo(area.TextElementNumber, 0);
+		cursor.moveToParagraph(area.ParagraphIndex);
+		cursor.moveTo(area.ElementIndex, 0);
 		const ZLTextWord &word = (ZLTextWord&)cursor.element();
-		ZLUnicodeUtil::Ucs2String ucs2;
-		ZLUnicodeUtil::utf8ToUcs2(ucs2, word.Data, word.Size);
-		ZLUnicodeUtil::Ucs2String::iterator it = ucs2.begin();
-		while ((it != ucs2.end()) && !ZLUnicodeUtil::isLetter(*it)) {
+		ZLUnicodeUtil::Ucs4String ucs4;
+		ZLUnicodeUtil::utf8ToUcs4(ucs4, word.Data, word.Size);
+		ZLUnicodeUtil::Ucs4String::iterator it = ucs4.begin();
+		while ((it != ucs4.end()) && !ZLUnicodeUtil::isLetter(*it)) {
 			++it;
 		}
-		if (it != ucs2.end()) {
-			ucs2.erase(ucs2.begin(), it);
-			it = ucs2.end() - 1;
+		if (it != ucs4.end()) {
+			ucs4.erase(ucs4.begin(), it);
+			it = ucs4.end() - 1;
 			while (!ZLUnicodeUtil::isLetter(*it)) {
 				--it;
 			}
-			ucs2.erase(it + 1, ucs2.end());
+			ucs4.erase(it + 1, ucs4.end());
     
-			ZLUnicodeUtil::ucs2ToUtf8(txt, ucs2);
+			ZLUnicodeUtil::ucs4ToUtf8(txt, ucs4);
 		}
 	}
 	return txt;
@@ -187,13 +187,16 @@ int FBView::bottomMargin() const {
 	return margins().BottomMarginOption.value();
 }
 
+/*
 ZLBooleanOption &FBView::selectionOption() {
 	if (ourSelectionOption.isNull()) {
 		ourSelectionOption = new ZLBooleanOption(ZLCategoryKey::LOOK_AND_FEEL, OPTIONS, "IsSelectionEnabled", true);
 	}
 	return *ourSelectionOption;
 }
+*/
 
 bool FBView::isSelectionEnabled() const {
-	return selectionOption().value();
+	//return selectionOption().value();
+	return false;
 }
