@@ -43,7 +43,7 @@ private:
 class FBIndicatorStyle : public ZLTextPositionIndicatorInfo {
 
 public:
-	ZLBooleanOption ShowOption;
+	ZLIntegerRangeOption TypeOption;
 	ZLBooleanOption IsSensitiveOption;
 	ZLBooleanOption ShowTextPositionOption;
 	ZLBooleanOption ShowTimeOption;
@@ -55,7 +55,7 @@ public:
 public:
 	FBIndicatorStyle();
 
-	bool isVisible() const;
+	Type type() const;
 	bool isSensitive() const;
 	bool isTextPositionShown() const;
 	bool isTimeShown() const;
@@ -70,12 +70,10 @@ class FBView : public ZLTextView {
 public:
 	static FBMargins& margins();
 	static FBIndicatorStyle& commonIndicatorInfo();
-	static ZLBooleanOption &selectionOption();
 
 private:
 	static shared_ptr<ZLTextPositionIndicatorInfo> ourIndicatorInfo;
 	static shared_ptr<FBMargins> ourMargins;
-	static shared_ptr<ZLBooleanOption> ourSelectionOption;
 
 protected:
 	void doTapScrolling(int y);
