@@ -26,6 +26,8 @@
 
 #include <ZLXMLReader.h>
 
+#include "../css/StyleSheetTable.h"
+
 class BookReader;
 class XHTMLReader;
 
@@ -56,9 +58,10 @@ public:
 	bool readFile(const std::string &pathPrefix, const std::string &fileName, const std::string &referenceName);
 	bool readFile(const std::string &pathPrefix, shared_ptr<ZLInputStream> stream, const std::string &referenceName);
 
+private:
 	void startElementHandler(const char *tag, const char **attributes);
 	void endElementHandler(const char *tag);
-	void characterDataHandler(const char *text, int len);
+	void characterDataHandler(const char *text, size_t len);
 
 	const std::vector<std::string> &externalDTDs() const;
 
@@ -67,10 +70,15 @@ private:
 	std::string myPathPrefix;
 	std::string myReferenceName;
 	bool myPreformatted;
+	bool myNewParagraphInProgress;
+	StyleSheetTable myStyleSheetTable;
+	std::vector<int> myCSSStack;
 
 	friend class XHTMLTagAction;
+	friend class XHTMLTagLinkAction;
 	friend class XHTMLTagHyperlinkAction;
 	friend class XHTMLTagPreAction;
+	friend class XHTMLTagParagraphAction;
 };
 
 #endif /* __XHTMLREADER_H__ */
