@@ -42,7 +42,6 @@
 #include "../fbreader/FBReader.h"
 #include "../fbreader/BookTextView.h"
 #include "../fbreader/CollectionView.h"
-#include "../fbreader/RecentBooksView.h"
 
 #include "../collection/BookCollection.h"
 #include "../external/ProgramCollection.h"
@@ -96,11 +95,11 @@ int RotationTypeEntry::initialCheckedIndex() const {
 	switch (myAngleOption.value()) {
 		default:
 			return 0;
-		case ZLViewWidget::DEGREES90:
+		case ZLView::DEGREES90:
 			return 1;
-		case ZLViewWidget::DEGREES180:
+		case ZLView::DEGREES180:
 			return 2;
-		case ZLViewWidget::DEGREES270:
+		case ZLView::DEGREES270:
 			return 3;
 		case -1:
 			return 4;
@@ -108,16 +107,16 @@ int RotationTypeEntry::initialCheckedIndex() const {
 }
 
 void RotationTypeEntry::onAccept(int index) {
-	int angle = ZLViewWidget::DEGREES0;
+	int angle = ZLView::DEGREES0;
 	switch (index) {
 		case 1:
-			angle = ZLViewWidget::DEGREES90;
+			angle = ZLView::DEGREES90;
 			break;
 		case 2:
-			angle = ZLViewWidget::DEGREES180;
+			angle = ZLView::DEGREES180;
 			break;
 		case 3:
-			angle = ZLViewWidget::DEGREES270;
+			angle = ZLView::DEGREES270;
 			break;
 		case 4:
 			angle = -1;
@@ -156,14 +155,8 @@ OptionsDialog::OptionsDialog(FBReader &fbreader) {
 	CollectionView &collectionView = fbreader.collectionView();
 	generalTab.addOption(ZLResourceKey("bookPath"), collectionView.collection().PathOption);
 	generalTab.addOption(ZLResourceKey("lookInSubdirectories"), collectionView.collection().ScanSubdirsOption);
-	RecentBooksView &recentBooksView = (RecentBooksView&)*fbreader.myRecentBooksView;
-	generalTab.addOption(ZLResourceKey("recentListSize"), new ZLSimpleSpinOptionEntry(recentBooksView.lastBooks().MaxListSizeOption, 1));
-	ZLToggleBooleanOptionEntry *showTagsEntry = new ZLToggleBooleanOptionEntry(collectionView.ShowTagsOption);
-	ZLOptionEntry *showAllBooksTagEntry = new ZLSimpleBooleanOptionEntry(collectionView.ShowAllBooksTagOption);
-	showTagsEntry->addDependentEntry(showAllBooksTagEntry);
-	generalTab.addOption(ZLResourceKey("showTags"), showTagsEntry);
-	generalTab.addOption(ZLResourceKey("showAllBooksList"), showAllBooksTagEntry);
-	showTagsEntry->onStateChanged(showTagsEntry->initialState());
+	generalTab.addOption(ZLResourceKey("recentListSize"), new ZLSimpleSpinOptionEntry(fbreader.recentBooks().MaxListSizeOption, 1));
+	generalTab.addOption(ZLResourceKey("showAllBooksTag"), collectionView.ShowAllBooksTagOption);
 
 	ZLDialogContent &encodingTab = myDialog->createTab(ZLResourceKey("Language"));
 	encodingTab.addOption(ZLResourceKey("autoDetect"), new ZLSimpleBooleanOptionEntry(PluginCollection::instance().LanguageAutoDetectOption));
@@ -176,8 +169,11 @@ OptionsDialog::OptionsDialog(FBReader &fbreader) {
 
 	myScrollingPage = new ScrollingOptionsPage(myDialog->createTab(ZLResourceKey("Scrolling")), fbreader);
 
-	ZLDialogContent &selectionTab = myDialog->createTab(ZLResourceKey("Selection"));
-	selectionTab.addOption(ZLResourceKey("enableSelection"), FBView::selectionOption());
+	//ZLDialogContent &selectionTab = myDialog->createTab(ZLResourceKey("Selection"));
+	//selectionTab.addOption(ZLResourceKey("enableSelection"), FBView::selectionOption());
+
+	ZLDialogContent &cssTab = myDialog->createTab(ZLResourceKey("CSS"));
+	cssTab.addOption(ZLResourceKey("overrideSpecifiedFonts"), ZLTextStyleCollection::instance().OverrideSpecifiedFontsOption);
 
 	ZLDialogContent &marginTab = myDialog->createTab(ZLResourceKey("Margins"));
 	FBMargins &margins = FBView::margins();
